/*
 edit.c - Main source file of SimpleEdit
 Date: 04/2000
 Author: maC (mac_mac@gmx.net)
 License: GPL
 Version: 0.5 beta
 Update: see www.geocities.com/boerdidrosi
*/

/*
  NOTE: since SimpleEdit is still BETA, and the sources might be a bit confusing. I'll order them soon. Maybe.
*/

/*
 v-pos:
	main()
	head
	InitDisplay()
*/

/*
TO DO:
   * Print cursor position at topright of window
   * Key-interrupt-handler
     => Enter ASCII-Characters by pressing the ALT-Key and some numbers
   * Tools:
     => ASM/C Syntax Highlighting
   * Eingabemodus "berschreiben" eincoden
   * Replace function
   * EnterString/LoadFile: don't destroy old File if operation is cancled
   * Implement TAB-characters
   * Mouse-Support

As soon as a better filesystem and a improved MemoryManagement is in the Kernel:
   * Rewrite Memory-Management 
     => Split Text into 64K-Blocks that can be spread around in MEM
     => Dynamic BackBuffer, with an auto-adapting size
   * Rewrite DiskIO
     => Rewrite save-function, allow overwriting of files
*/

#include "v2os.h"
#include "edit.h"

/****** LINE AND TEXT MEMORY BUFFERS **************/
int LB_ptr;	//LineBuffer
int LB_size;

int TB_size;	//TextBuffer
int TB_ptr;

#define BB_MaxSize 1024
int BB_size;	//BufferBuffer :)
int BB_ptr;
int BB_UsedSize;

/******* LINE AND TEXT INFORMATIONS ****************/
#define MaxLines (8192)
int line_count;

#define SecurityDistance (32)

int text_size;
int text_style; //0=DOS, 1=UNIX

int Mark_Start;
int Mark_End;

/******* FILE INFORMATION **************/
char file_name[100] = "";
int file_disk;
int file_data_start;
int file_sector_count;
int file_attributes;
int file_rawlength;


/******* DISKBUFFER INFORMATION ************/
int DB_size=0;
int DB_offs=0;


/******* WINDOW DISPLAY INFORMATION ****************/
int w_x,w_y,w_w,w_h;	// Screenposition of text-window
int t_x,t_y;	// Position of Text inside the Window
int c_x,c_y;	// Position of Cursor inside the Text

char* Status_text="-";
int Status_color=0;

#define status_color_error (0+16*6)
#define status_color_normal (1+16*4)

int ScreenSaver;
int ScreenSaverSize;

/********** MOUSE INFORMATION **************************/
char* Mouse_ID = "V2_MOUSE";
int Mouse_Module; //0 if no mousemodule is installed, <>0 otherwise
int Mouse_X;
int Mouse_Y;
short Mouse_Buttons;

#define ME_Lost 0
#define ME_LButtonDown 1
#define ME_LButtonUp 2
#define ME_RButtonDown 3
#define ME_RButtonUp 4
#define ME_Move 5

/********** TEXT AND LINE BUFFER ACCSSES ******************/
#define GetLineStart(a) peekl(0x18,((a)<<3)+LB_ptr)
#define GetLineLength(a) peekl(0x18,((a)<<3)+LB_ptr+4)
#define SetLineStart(a,b) pokel(0x18,((a)<<3)+LB_ptr,b)
#define SetLineLength(a,b) pokel(0x18,((a)<<3)+LB_ptr+4,b)

#define GetChar(a) peekb(0x18,TB_ptr+(a))
#define SetChar(a,b) pokeb(0x18,TB_ptr+(a),b);

void inline PutChar (int x, int y, char c, int col)
{
	pokeb( 0x18, 0xB8000+  ((x+y*80)<<1), c );
	pokeb( 0x18, 0xB8000+1+((x+y*80)<<1), col );
}

void Window( int x, int y, int w, int h, int col, unsigned char* title, int ScrollBar, int bottom)
{
	int i,j;

	for( i=x+1; i<x+w; i++ )
	{
		PutChar( i, y, 196, col );
		if( bottom==1) PutChar( i, y+h, 196, col );
	}
	if( ScrollBar==0 ) j=179; else j=186;
	for( i=y+1; i<y+h; i++)
	{
		PutChar( x, i, 179, col);
		PutChar( x+w, i, j, col); //scroll-bar
	}
	PutChar( x, y, 218, col );
	PutChar( x+w, y, 191, col );
	PutChar( x, y+h, 192, col );
	PutChar( x+w, y+h, 217, col );
	i=v2_strlen(title);

		//Print Title (if any)
	if( i>0 )
	{
		PutChar( x+2, y, ' ', col);
		for( j=0; j<i; j++ )
			PutChar(x+3+j, y, *title++, col);
		PutChar( x+3+j, y, ' ', col);
	}

		// Print Cursor position

		// Scroll-Bar
	if( ScrollBar==1 )
	{
		PutChar( x+w, y+1, 197, col );
		PutChar( x+w, y+h-1, 197, col );
	}
}

void SetScrollBar( int x, int y, int w, int h, int col, int pos, int max )
{
	int v;		// = ((h-4)*(pos))/(max-1);	// pos<max!!, pos[0] = 0!!
	if( (max-1)==0 ) v=0;
		else v=((h-4)*(pos))/(max-1);

/*	if( max==0 ) v=0;
	else v=((h-4)*(pos))/max;
*/
	if( v>h-4 ) v=h-4;
	PutChar( x+w, y+2+v, 219, col );
}

#include "FileIO.c"
#include "menu.c"
#include "Textuse.c"
#include "memman.c"
#include "mouse.c"


void InputLoop()
{
	unsigned char c;
	int state=1;
	int update=1;

/*	int ctrl_down;
	int alt_down;
	int shift_down;

	char c2;
*/
	int Event;
	ProcessMenus();

	for(;;)
	{
		if( update==1 )
		switch( state )
		{
		case 1: Display(1); break;
		case 0: Display(2); break;
		}

/*
		c = getkey();
		if( c==27 ) getkey(); // BUGFIX: V2OS returns 2 characters, if ESC is pressed. we have to catch the second
*/

		Event=0;
		for(;;)
		{
			c=getkey_nowait();
			if( c!=0 ) { Event=1; break; }
			if( (UpdateMousePos()==1)&(Mouse_Buttons!=0) ) {Event=2; break; }
			//Display(0);
			if( UpdateMouseButtons()!=0 ) {Event=3; break;}
		}
		if( c==27 ) getkey();

		update=1;

		if( Event==1 )
		{
			switch( state )
			{
			case 0: switch(MakeMenuChar( c ))
				{
				case 0: return;
				case 1: state=1; break;
				case 3: Display_Mouse(1);
					DisplayMainMenu(mx, my);
					update=0; //no need to redraw all (avoids flickering)
					Display_Mouse(0);
					break;
				}
				break;
			case 1: switch( ProcessEditChar( c ))
				{
				case 0: state=0; 
					mx=0;
					my=0;
					break;
				}
				break;
			}
		} else //Not a keypress but a mouseklick
		{
			switch( state )
			{
			case 0: switch( MakeMenuKlick( Event ) )
				{
				case 11:
				case 12:
				case 5: state=1; break;

/*				case 5: Error(" You've klicked a 5!"); break;
				case 1: Error(" You've klicked a 1!"); break;
				case 2: Error(" You've klicked a 2!"); break;
*/
				case 10: return; //quit
				}
				break;
			case 1: if( (Mouse_Y>0) | (Event==2) )
					ProcessEditMouse( Event );
				if( Mouse_Y==0 )
				{
					if( MakeMenuKlick( Event )==4 )
						state=0;
				}
				break;
			}
		} // mouseklick
	}
}

/*		alt_down = KeyDown( 56 );
		ctrl_down = KeyDown( 29 );
		shift_down = KeyDown( 42 );
*/
/*
		if( KeyDown(42)==1 ) c2='D'; else c2='U';
		PutChar( 3, 51, c2, 7 ); //left shift
		if( KeyDown(54)==1 ) c2='D'; else c2='U';
		PutChar( 4, 51, c2, 7 ); // right shift
		if( KeyDown(56)==1 ) c2='D'; else c2='U';
		PutChar( 6, 51, c2, 7 ); // alt
		if( KeyDown(29)==1 ) c2='D'; else c2='U';
		PutChar( 8, 51, c2, 7 ); // CTRL
*/

int main(void)
{
	term_puts( SCR_DEBUG, "\r\n=== SimpleEdit starting here ===\r\n" );

	puts("V2_SimpleEdit     v0.5 (beta)     4/2000      written by maC (mac_mac@gmx.net), published under GPL\r\n");
	puts(" see http://www.geocities.com/boerdidrosi for updates\r\n\r\n");

	GetMemory( 1024 );
	Text_New();

	Mouse_Init();

	InitDisplay();

	 term_puts( SCR_DEBUG, "Starting InputLoop()...\r\n");
	InputLoop();

	RestoreDisplay();

	puts("\r\n");

	 term_puts( SCR_DEBUG, "=== SimpleEdit ending here ===\r\n" );
	return 0;
}
